C-----------------------------------------------------------------------
C
CN  ROUTINE Convert ARC coverage to footprint file
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
CP  Purpose:
C     This routine extracts the external (universe) polygon from a coverage
C     and writes it into a textfile for use in spatial indexing.
CE
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
CA  Arguments:
C     COVER   ==  (Character*128) The coverage to be converted.
C     EXPORT  ==  (Character*128) The pathname of the file to be
C                  created.
C     OUTFORM ==  (INTEGER) A flag to put file out in:
C                  0 = Arc GENERATE format
C                  1 = DSDL Footprint format
CE
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
CH  History:
C
C    Doug Nebert, USGS             8/13/92            Modified from arcmoss code
C    (Peter Aronson                06/09/88           Original coding.)
CE
C-----------------------------------------------------------------------
C
      PROGRAM FOOTPRINT
C
      INTEGER  IERROR,KANATT,KANEXP,IFLAG(500),LENGTH,TOTPNT,OUTFORM
      INTEGER  KANDMP,KANPAL,KANARC,IPOL,IARC,I,J,L,NPOLY     
      CHARACTER*128  COVER,EXPORT,TMPFIL
      CHARACTER*30   ATTRIB,LINE,LINEX
      LOGICAL  QJUMP
      DOUBLE PRECISION  LASTX,LASTY
      EXTERNAL  PALOPN,NONFTL,ARCOPN,SCRNAM,ACREAT,PALRD,GETSTR,MESINT
      EXTERNAL  LUNINI,MESINI,MINIT,VINIT,INFINT,GETINT,ARGRCV,ARGRRE
      EXTERNAL  ARGRCH,ARGRIN,ARGEND,ARCMOS,AEXIT,AENTER
      EXTERNAL  GETOPN,POLMOS,LINMOS,PNTMOS,ACLOSE,ADELET
      EXTERNAL  MESCHR,GETCLS
      EXTERNAL  PALCLS,ARCCLS,WARN,PALREC,ARCRDR,FATAL
      DATA  IFLAG /500*0/
      DATA  ATTRIB /' '/
C.. Initialize subsystems.
C
      include '/arc50/brksys/brkset.inc'
      include '/arc50/inserts/palrec.def'
      include '/arc50/inserts/arcrec.def'
C
      CALL LUNINI
      CALL MESINI
      CALL MINIT
      CALL VINIT
      CALL INFINT
      CALL GETINT
C
C.. Read in arguments.
C
      READ(*,*) COVER
      READ(*,*) EXPORT
      READ(*,*) OUTFORM
C
C.. FORMAT FOR OUTFORM: 0 = generate format
C                       1 = footprint format
C
C.. Create export file.
C
      CALL ACREAT (KANEXP,EXPORT,IERROR)
      IF (IERROR .NE. 0) THEN
        CALL MESCHR (EXPORT,0)
        CALL NONFTL ('Unable to create %1%.',-1)
        IERROR = -2
        RETURN
      ENDIF
C
C.. Open the PAL file.
C
      CALL PALOPN (KANPAL,COVER,1,1,IERROR)
      IF (IERROR .NE. 0) THEN
        CALL NONFTL ('Unable to open PAL file.',-1)
        IERROR = -1
        RETURN
      ENDIF
C
C.. Open the ARC file.
C
      CALL ARCOPN (KANARC,COVER,1,1,IERROR)
      IF (IERROR .NE. 0) THEN
        CALL NONFTL ('Unable to open ARC file.',-1)
        IERROR = -3
        RETURN
      ENDIF
C
C.. Create the scratch file.
C
      CALL SCRNAM (TMPFIL)
      CALL ACREAT (KANDMP,TMPFIL,IERROR)
      IF (IERROR .NE. 0) THEN
        CALL NONFTL ('Unable to create polygon scratch file.',-1)
        IERROR = -4
        CALL PALCLS (KANPAL)
        CALL ARCCLS (KANARC)
        RETURN
      ENDIF
C
C.. Read in the PAL record.
C
 10   CALL PALRD (KANPAL,IPOL,IPBUFF,IERROR)
      IF (IERROR .NE. 0) GOTO 900
C
C.. Rewind the scratch file and initialize status variables.
C
      REWIND (KANDMP,ERR=800)
      TOTPNT = 0
      QJUMP = .FALSE.
C
C.. Read through the Arcs in the polygon, writing their coordinates
C.. out to the dump file.
C
      DO 11 I = 1,NPAL
        IARC = IPAL(1,I)
        IF (IARC .NE. 0) THEN
          CALL ARCRDR (KANARC,ABS(IARC),IABUFF,IERROR)
          IF (IERROR .NE. 0) CALL FATAL ('Bad PAL or ARC file.',20)
          TOTPNT = TOTPNT + NPNTS
        ELSE
          NPNTS = 0
        ENDIF
        IF (QJUMP .AND. IARC .NE. 0) THEN
            IF (IARC .GT. 0) THEN
              IFLAG(1) = 1
            ELSE
              IFLAG(NPNTS) = 1
            ENDIF
          QJUMP = .FALSE.
          TOTPNT = TOTPNT + 1
        ENDIF
        IF (IARC .GT. 0) THEN
          DO 12 J = 1,NPNTS
            IF (OUTFORM .EQ. 1) THEN
              WRITE(KANDMP,2002,ERR=800) (PNTS(2,J),',',PNTS(1,J))
            ELSE
              WRITE(KANDMP,2002,ERR=800) (PNTS(1,J),',',PNTS(2,J))
            ENDIF
 12       CONTINUE
 2002       FORMAT(F10.5,A1,F10.5)
            IFLAG(1) = 0
          LASTX = PNTS(1,NPNTS)
          LASTY = PNTS(2,NPNTS)
        ELSEIF (IARC .LT. 0) THEN
          DO 13 J = NPNTS,2,-1
            IF (OUTFORM .EQ. 1) THEN
              WRITE(KANDMP,2002,ERR=800) (PNTS(2,J),',',PNTS(1,J))
            ELSE
              WRITE(KANDMP,2002,ERR=800) (PNTS(1,J),',',PNTS(2,J))
            ENDIF
 13       CONTINUE
          LASTX = PNTS(1,1)
          LASTY = PNTS(2,1)
          IFLAG(NPNTS) = 0
        ELSE
          QJUMP = .TRUE.
        ENDIF
 11   CONTINUE
CC        IF (OUTFORM .EQ. 1) THEN
CC          WRITE (KANDMP,2002) LASTY,',',LASTX
CC        ELSE
CC          WRITE (KANDMP,2002) LASTX,',',LASTY
CC        ENDIF
CC      TOTPNT = TOTPNT + 1
C
C.. Write out polygon header.
C
      IF (OUTFORM .EQ. 1) THEN
        write (kanexp,*) 'inside('
      ELSE
        write (KANEXP,*) '1'
      ENDIF
C
C.. Write out the coordinates.
C
      REWIND (KANDMP,ERR=800)
	K = 0
 20   READ (KANDMP,'(A)',END=900) LINE
	K = K + 1
	IF (K .EQ. 1) THEN
 	  LINEX = LINE
	ENDIF
      WRITE (KANEXP,'(A)',ERR=800) LINE
	IF ((K .GT. 1) .AND. (LINEX .EQ. LINE)) THEN
	   IF (OUTFORM .EQ. 1) THEN
		WRITE (KANEXP,*) ')'
	  	WRITE (KANEXP,*) 'inside('
	   	K = 0
	   ELSE
		WRITE (KANEXP,*) 'END'
		WRITE (KANEXP,*) '1'
		K = 0
	   ENDIF	  
	ENDIF
      GOTO 20
C
C.. Error exit.
C
 800  IERROR = -5
      CALL MESINT (IPOL)
      CALL NONFTL ('Error in writing Polygon #%1%, Conversion aborted.'
     +            ,-1)
      GOTO 901
C
C
C      CALL AEXIT
C
C.. Close up.
C
 900  CONTINUE
      IF (OUTFORM .EQ. 1) THEN
         write (kanexp,*) ')'
      ELSE
         WRITE (KANEXP,*) 'END'
         WRITE (KANEXP,*) 'END'
      ENDIF
      IERROR = 0
 901  CALL ACLOSE (KANEXP)
      CALL ACLOSE (KANDMP)
      CALL ADELET (TMPFIL,IERROR)
      CALL PALCLS (KANPAL)
      CALL ARCCLS (KANARC)
      IF (IERROR .NE. 0) THEN
        CALL ADELET (EXPORT,IERROR)
        IERROR = -3
        RETURN
      ENDIF
C
C
      IERROR = 0
      RETURN
      END
